<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined('JPATH_PLATFORM') or die;

jimport('noboss.util.curl');

// Classe para requisicoes para API do Google Calendar
class NoBossApiGooglecalendar {

    private $client;
    private $redirectURI;
    private $config;
    private $scope = 'https://www.googleapis.com/auth/calendar.readonly';
    private $oauthURL = 'https://accounts.google.com/o/oauth2/v2/auth';
    private $oauthTokenURL = 'https://oauth2.googleapis.com/token';

    /**
     * __construct
     *
     * @param   string  $redirectURI    url para redirecionar apos autorizar aplicacao (deve estar tb cadastrada no console da API)
     * @param   array   $config         matriz associativa, deve conter client_id e client_secret.
     * @return void
     */
    public function __construct($redirectURI, $config) {
        $this->redirectURI = $redirectURI;
        $this->config = $config;
    }

    /**
     * Autoriza o usuario junto a API obtendo o token para executar proximos metodos
     *
     * @return string|null  access_token    necessário para outros métodos
     */
    public function authorize() {
        if (!isset($_GET['code'])) {
            $authURL = "{$this->oauthURL}?" . \http_build_query([
                'scope' => $this->scope,
                'access_type' => 'offline',
                'include_granted_scopes' => 'true',
                'response_type' => 'code',
                'redirect_uri' => $this->redirectURI,
                'client_id' => $this->config['client_id']
            ]);
            header('Location: ' . filter_var($authURL, FILTER_SANITIZE_URL));
        } else {
            $authCode = $_GET['code'];
            $accessToken = $this->authenticate($authCode)['access_token'];
            return $accessToken;
        }
    }

    /**
     * Autentique o código de troca para a estrutura access_code.
     *
     * @param   string  $code   returned by authorize()
     * 
     * @return  array access and refresh token
     */
    private function authenticate($code) {
        $queryParams = [
            'client_id' => $this->config['client_id'],
            'client_secret' => $this->config['client_secret'],
            'code' => $code,
            'grant_type' => 'authorization_code',
            'redirect_uri' => $this->redirectURI
        ];
        $response = NoBossUtilCurl::request("POST", $this->oauthTokenURL, $queryParams);
        
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }
            
        return \json_decode($response->data, true);
    }

    /**
     * Obtem lista de calendarios vinculados a um token
     *
     * @param   string      $token      access_token
     * 
     * @return  array       Dados dos calendarios vinculados ao token
     */
    public function getCalendars($token) {
        $dest = 'https://www.googleapis.com/calendar/v3/users/me/calendarList';
        $queryParams = [
            'maxResults' => 50,
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];

        $items = [];

        // lista pode retornar varias páginas, fazemos um loop até $pageToken não ser válido
        while (true) {
            $response = NoBossUtilCurl::request("GET", "{$dest}?" . \http_build_query($queryParams), null, $headers);
            
            if (!$response->success) {
                $data = json_decode($response->data);
                throw new \Exception($data->error->message, $data->error->code);
            }

            $json = \json_decode($response->data, true);


            foreach ($json['items'] as $item) {
                $items[] = $item;
            }

            if (\array_key_exists('nextPageToken', $json)) {
                $queryParams['pageToken'] = $json['nextPageToken'];
            } else {
                break;
            }
        }
        return $items;
    }

    /**
     * Obtem a lista de eventos vinculados a um calendario dentro de um periodo de tempo
     *
     * @param   string      $token          access_token
     * @param   string      $calendarID     Id do calendario
     * @param   string      $start          lower bound (exclusive) RFC3339 timestamp + timezone. Eg: '2020-05-03T00:00:00+00:00'
     * @param   string      $end            upper bound (exclusive) RFC3339 timestamp + timezone. Eg: '2020-05-09T00:00:00+00:00'
     * 
     * @return  array of events inside $calendarID and between $start and $end
     */
    public function getCalendarEvents($token, $calendarID, $start, $end) {
        $dest = "https://www.googleapis.com/calendar/v3/calendars/" . \rawurlencode($calendarID) . "/events";
        $queryParams = [
            'maxResults' => 50,
            'timeMin' => $start,
            'timeMax' => $end
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];

        $items = [];

        // lista pode retornar varias páginas, fazemos um loop até $pageToken não ser válido
        while (true) {
            $response = NoBossUtilCurl::request("GET", filter_var("{$dest}?" . \http_build_query($queryParams), FILTER_SANITIZE_URL), null, $headers);
           
            if (!$response->success) {
                $data = json_decode($response->data);
                throw new \Exception($data->error->message, $data->error->code);
            }

            $json = \json_decode($response->data, true);

            foreach ($json['items'] as $item) {
                $items[] = $item;
            }

            if (\array_key_exists('nextPageToken', $json)) {
                $queryParams['pageToken'] = $json['nextPageToken'];
            } else {
                break;
            }
        }
        return $items;
    }
}
