<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined('JPATH_PLATFORM') or die;

jimport('noboss.util.curl');

// Classe para requisicoes para API do Vimeo
class NoBossApiVimeo {

    private $redirectURI;
    private $client;
    private $config;
    private $scope = "public private";
    private $oauthURL = 'https://api.vimeo.com/oauth/authorize';
    private $oauthTokenURL = 'https://api.vimeo.com/oauth/access_token';

    /**
     * __construct
     *
     * @param   string      $redirectURI    url para redirecionar apos autorizar aplicacao (deve estar tb cadastrada no console da API)
     * @param   array       $config         matriz associativa, deve conter client_id e client_secret.
     * 
     * @return void
     */
    public function __construct($redirectURI, $config) {
        $this->redirectURI = $redirectURI;
        $this->config = $config;
    }

    /**
     * Autoriza o usuario junto a API obtendo o token para executar proximos metodos
     *
     * @return string|null access_token needed for other methods
     */
    public function authorize() {
        if (!isset($_GET['code'])) {
            $authURL = "{$this->oauthURL}?" . \http_build_query([
                'response_type' => 'code',
                'client_id' => $this->config['client_id'],
                'redirect_uri' => $this->redirectURI,
                'state' => '1',
                'scope' => $this->scope
            ]);
            header('Location: ' . filter_var($authURL, FILTER_SANITIZE_URL));
        } else {
            $authCode = $_GET['code'];
            $accessToken = $this->authenticate($authCode)['access_token'];
            return $accessToken;
        }
    }

    /**
     * Autentique o codigo de troca para a estrutura access_code.
     *
     * @param   string  $code   returned by authorize()
     * 
     * @return  array access and refresh token
     */
    private function authenticate($code) {
        $headers = [
            'Authorization' => 'basic ' . \base64_encode("{$this->config['client_id']}:{$this->config['client_secret']}"),
            'Content-Type' => 'application/json',
            'Accept' => 'application/vnd.vimeo.*+json;version=3.4'
        ];

        $queryParams = [
            'grant_type' => 'authorization_code',
            'code' => $code,
            'redirect_uri' => $this->redirectURI
        ];
        
        $response = NoBossUtilCurl::request("POST", $this->oauthTokenURL, \json_encode($queryParams), $headers);

        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error, 1);
        }
            
        return \json_decode($response->data, true);
    }

    /**
     * Obtem playlists disponiveis ao token informado
     *
     * @param   string    $token  access_token
     * 
     * @return array with each playlist information
     */
    public function getPlaylists($token) {
        $dest = 'https://api.vimeo.com/me/albums';

        $queryParams = [
            'per_page' => 100,
            'page' => 1
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];
        $playlists = [];
        
        while (true) {
            $response = NoBossUtilCurl::request("GET", "{$dest}?" . http_build_query($queryParams), null, $headers);

            if (!$response->success) {
                $data = json_decode($response->data);
                throw new \Exception($data->error, 1);
            }

            $json = \json_decode($response->data, true);

            foreach ($json['data'] as $playlistItem) {
                // Obtem o id do video a partir do link do album
                preg_match('/\/(\d+)$/', $playlistItem['share_link'], $match);
                $playlistItem['id_video'] = $match[1];

                 // FIXME: avaliar se nao vale a pena aproveitar que estou fazendo um foreach para preparar melhor os dados a serem retornados

                $playlists[] = $playlistItem;
            }

            if (! $json['paging']['next']) {
                break;
            } else {
                $query['page']++;
            }
        }

        return $playlists;
    }

    /**
     * Lista informacoes dos videos de uma playlist
     *
     * @param   string      $token                  access_token
     * @param   string      $playlistID             id of playlist
     * @param   Boolean     $displayPlaylistData    informa se deve ser retornado tb os dados da playlist
     * @param   string      $direction              ordenacao por data do video (asc|desc)
     * 
     * @return array with videos found at $plalistID
     */
    public function getVideosPlaylist($token, $playlistID, $direction = 'asc', $displayPlaylistData = false) {
        $dest = "https://api.vimeo.com/me/albums/{$playlistID}/videos";

        $queryParams = [
            'per_page' => 100,
            'page' => 1,
            'sort' => 'date',
            'direction' => $direction
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];
        
        $return = new stdClass();
        $return->videos = [];

        // Setado para obter dados da playlist tb
        if($displayPlaylistData){
            $return->playlist = self::getPlaylistData($token, $playlistID);
        }
        
        while (true) {
            $response = NoBossUtilCurl::request("GET", "{$dest}?" . http_build_query($queryParams), null, $headers);
            
            if (!$response->success) {
                $data = json_decode($response->data);
                throw new \Exception($data->error, 1);
            }

            $json = \json_decode($response->data, true);

            // Percorre todos videos da playlist
            foreach ($json['data'] as $video) {
                // Obtem o id do video a partir do link
                preg_match('/\/(\d+)$/', $video['link'], $match);
                $video['id_video'] = $match[1];
                
                // FIXME: avaliar se nao vale a pena aproveitar que estou fazendo um foreach para preparar melhor os dados a serem retornados
                
                // Dados do video
                $return->videos[] = $video;
            }

            // $return->videos[] = $json['data'];

            if (! $json['paging']['next']) {
                break;
            } else {
                $query['page']++;
            }
        }

        return $return;
    }

    /**
     * Obtem dados de uma playlist
     *
     * @param   string      $token          access_token
     * @param   string      $idPlaylist     Id da playlist
     * 
     * @return  array       Dados da playlist
     */
    private function getPlaylistData($token, $idPlaylist) {
        $dest = "https://api.vimeo.com/me/albums/{$idPlaylist}";

        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];
        
        $response = NoBossUtilCurl::request("GET", "{$dest}", null, $headers);
        
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error, 1);
        }

        $json = \json_decode($response->data, true);

        return $json;
    }

    /**
     * Obtem detalhes de um video
     *
     * @param   string      $token access_token
     * @param   string      $videoID id of video
     * 
     * @return  array       with details of $videoID
     */
    public function getVideoData($token, $videoID) {
        $dest = "https://api.vimeo.com/videos/{$videoID}";

        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];
        
        $response = NoBossUtilCurl::request("GET", "{$dest}", null, $headers);
        
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error, 1);
        }

        $json = \json_decode($response->data, true);

        return $json;
    }
}
