<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined('JPATH_PLATFORM') or die;

jimport('noboss.util.curl');

// Classe para requisicoes para API do Youtube
class NoBossApiYoutube {

    private $client;
    private $redirectURI;
    private $config;
    private $scope = 'https://www.googleapis.com/auth/youtube.readonly';
    private $oauthURL = 'https://accounts.google.com/o/oauth2/v2/auth';
    private $oauthTokenURL = 'https://oauth2.googleapis.com/token';

    /**
     * __construct
     *
     * @param   string      $redirectURI    url para redirecionar apos autorizar aplicacao (deve estar tb cadastrada no console)
     * @param   array       $config         matriz associativa, deve conter client_id e client_secret.
     * 
     * @return void
     */
    public function __construct($redirectURI, $config) {
        $this->redirectURI = $redirectURI;
        $this->config = $config;
    }

    /**
     * Autoriza o usuario junto a API obtendo o token para executar proximos metodos
     *
     * @return string|null  access_token    necessário para outros métodos
     */
    public function authorize() {
        if (!isset($_GET['code'])) {
            $authURL = "{$this->oauthURL}?" . \http_build_query([
                'scope' => $this->scope,
                'access_type' => 'offline',
                'include_granted_scopes' => 'true',
                'response_type' => 'code',
                'redirect_uri' => $this->redirectURI,
                'client_id' => $this->config['client_id']
            ]);
            header('Location: ' . filter_var($authURL, FILTER_SANITIZE_URL));
        } else {
            $authCode = $_GET['code'];
            $accessToken = $this->authenticate($authCode)['access_token'];
            return $accessToken;
        }
    }

    /**
     * Autentique o código de troca para a estrutura access_code.
     *
     * @param   string  $code   returned by authorize()
     * 
     * @return  array access and refresh token
     */
    private function authenticate($code) {
        $queryParams = [
            'client_id' => $this->config['client_id'],
            'client_secret' => $this->config['client_secret'],
            'code' => $code,
            'grant_type' => 'authorization_code',
            'redirect_uri' => $this->redirectURI
        ];
        $response = NoBossUtilCurl::request("POST", $this->oauthTokenURL, $queryParams);

        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }
            
        return \json_decode($response->data, true);
    }

    /**
     * Obtem playlists disponiveis ao token informado
     *
     * @param   string    $token  access_token
     * 
     * @return array with each playlist information
     */
    public function getPlaylists($token) {
        $dest = 'https://www.googleapis.com/youtube/v3/playlists';
        $queryParams = [
            'part' => 'snippet',
            'maxResults' => 50,
            'mine' => true
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];

        $items = [];

        // lista pode retornar varias páginas, fazemos um loop até $pageToken não ser válido
        while (true) {
            $response = NoBossUtilCurl::request("GET", "{$dest}?" . \http_build_query($queryParams), null, $headers);

            if (!$response->success) {
                $data = json_decode($response->data);
                throw new \Exception($data->error->message, $data->error->code);
            }

            $json = \json_decode($response->data, true);

            foreach ($json['items'] as $item) {
                // FIXME: avaliar se nao vale a pena aproveitar que estou fazendo um foreach para preparar melhor os dados a serem retornados
                $items[] = $item;
            }

            if (\array_key_exists('nextPageToken', $json)) {
                $queryParams['pageToken'] = $json['nextPageToken'];
            } else {
                break;
            }
        }
        return $items;
    }

    /**
     * Lista informacoes dos videos de uma playlist
     *
     * @param   string      $token                  access_token
     * @param   string      $playlistID             id of playlist
     * @param   Boolean     $displayPlaylistData    informa se deve ser retornado tb os dados da playlist
     * @param   string      $direction              ordenacao por data do video (asc|desc)
     * 
     * @return array with videos found at $plalistID
     */
    public function getVideosPlaylist($token, $playlistID, $direction = 'asc', $displayPlaylistData = false) {
        $dest = 'https://www.googleapis.com/youtube/v3/playlistItems';
        $queryParams = [
            'part' => 'snippet',
            'maxResults' => 50,
            'playlistId' => $playlistID
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];

        $return = new stdClass();
        $return->videos = [];

        // Setado para obter dados da playlist tb
        if($displayPlaylistData){
            $return->playlist = self::getPlaylistData($token, $playlistID);
        }

        while (true) {
            $response = NoBossUtilCurl::request("GET", "{$dest}?" . http_build_query($queryParams), null, $headers);
            if (!$response->success) {
                $data = json_decode($response->data);
                throw new \Exception($data->error->message, $data->error->code);
            }

            $json = \json_decode($response->data, true);

            // Percorre todos videos da playlist
            foreach ($json['items'] as $video) {
                // Dados do video
                // FIXME: avaliar se nao vale a pena aproveitar que estou fazendo um foreach para preparar melhor os dados a serem retornados
                $return->videos[] = $video;
            }

            if (\array_key_exists('nextPageToken', $json)) {
                $queryParams['pageToken'] = $json['nextPageToken'];
            } else {
                break;
            }
        }

        usort($return->videos, function ($a, $b) use ($direction) {
            if ($direction == 'asc') {
                return strtotime($a['snippet']['publishedAt']) < strtotime($b['snippet']['publishedAt']) ? -1 : 1;
            }
            return strtotime($a['snippet']['publishedAt']) > strtotime($b['snippet']['publishedAt']) ? -1 : 1;
        });

        return $return;
    }

    /**
     * Obtem dados de uma playlist
     *
     * @param   string      $token          access_token
     * @param   string      $idPlaylist     Id da playlist
     * 
     * @return  array       Dados da playlist
     */
    private function getPlaylistData($token, $idPlaylist) {
        
        $dest = 'https://www.googleapis.com/youtube/v3/playlists';
        $queryParams = [
            'part' => 'snippet',
            'maxResults' => 1,
            'id' => $idPlaylist
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];

        $items = [];

        $response = NoBossUtilCurl::request("GET", "{$dest}?" . \http_build_query($queryParams), null, $headers);
       
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }

        $json = \json_decode($response->data, true);

        return $json['items'][0]['snippet'];
    }

    /**
     * Obtem detalhes de um video
     *
     * @param   string      $token access_token
     * @param   string      $videoID id of video
     * 
     * @return  array       with details of $videoID
     */
    public function getVideoData($token, $videoID) {
        $dest = 'https://www.googleapis.com/youtube/v3/videos';

        $queryParams = [
            'part' => 'snippet,contentDetails',
            'id' => $videoID
        ];
        $headers = [
            'Authorization' => "Bearer {$token}",
            'Accept' => 'application/json'
        ];

        $response = NoBossUtilCurl::request("GET", "{$dest}?" . http_build_query($queryParams), null, $headers);

        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }

        $json = \json_decode($response->data, true);

        // Video nao encontrado
        if (!$json['items']) {
            return false;
        }

        return $json['items'][0];
    }
}
